using UnityEngine;
using System.Text;
using System.Collections;
using System.Globalization;

namespace System
{
	public static class DateTimeExtensions
	{
		public static string ToRFC822String(this DateTime timestamp)
		{
			return timestamp.ToString("ddd',' d MMM yyyy HH':'mm':'ss")
				+ " "
				+ timestamp.ToString("zzzz").Replace(":", "");
		}

		public static string ToRFC822StringAlternate(this DateTime timestamp)
		{
			var builder = new StringBuilder(timestamp.ToString("ddd, dd MMM yyyy HH:mm:ss zzz", CultureInfo.InvariantCulture));
			builder.Remove(builder.Length - 3, 1);
			return builder.ToString();
		}

		public static string ToRFC822OffsetString(this DateTime timestamp)
		{
			string rfc822 = timestamp.ToRFC822String();
			if (rfc822.Length > 5)
			{
				return rfc822.Substring(rfc822.Length - 5);
			}
			return string.Empty;
		}

		/// <summary>
		/// Waits until this dateTime is reached
		/// </summary>
		/// <param name="dateTime">Date time after which this method will stop waiting. Must be supplied in utc for performance reasons.</param>
		/// <param name="callback">An optional callback providing the up-to-date time span between now and the target time</param>
		public static IEnumerator Wait(this DateTime dateTime, float tickDuration = 1, Action<TimeSpan> onTick = null, Action onCompleted = null)
		{
			DateTime now;
			DateTime target = dateTime.ToUniversalTime();
			YieldInstruction wait = new WaitForSeconds(tickDuration);

			while (DateTime.Compare(now = DateTime.UtcNow, target) < 0)
			{
				if (onTick != null)
				{
					onTick(target.Subtract(now));
				}
				yield return wait;
			}

			if (onTick != null)
			{
				onTick(TimeSpan.Zero);
			}

			if (onCompleted != null)
			{
				onCompleted();
			}
		}
	}
}
