﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;


public static class EnumExtensions
{
	// http://wiki.unity3d.com/index.php?title=EnumExtensions
	public static bool TryParse<T>(this Enum theEnum, string valueToParse, out T returnValue)
	{
		returnValue = default(T);
		if (Enum.IsDefined(typeof(T), valueToParse))
		{
			TypeConverter converter = TypeDescriptor.GetConverter(typeof(T));
			returnValue = (T)converter.ConvertFromString(valueToParse);
			return true;
		}
		return false;
	}

	// http://stackoverflow.com/questions/4108828/generic-extension-method-to-see-if-an-enum-contains-a-flag
	public static bool HasFlag(this Enum variable, Enum value)
	{
		if (variable == null)
		{
			// don't throw an argument null exception; a null value just isn't present.
			return false;
		}

		if (value == null)
		{
			throw new ArgumentNullException ("value");
		}

		var variableType = variable.GetType();
		if (!Enum.IsDefined(variableType, value))
		{
			var message = string.Format("Enumeration type mismatch.  The flag is of type '{0}', was expecting '{1}'.",
			                            value.GetType(), variableType);
			throw new ArgumentException (message, "value");
		}

		ulong num = Convert.ToUInt64(value);
		return ((Convert.ToUInt64(variable) & num) == num);
	}

	// http://damieng.com/blog/2008/04/10/using-linq-to-foreach-over-an-enum-in-c
	public static IEnumerable<T> GetEnumerable<T>()
	{
		return Enum.GetValues(typeof(T)).Cast<T>();
	}
}
