using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace UnityEngine
{
	public static class GameObjectExtensions
	{
		public static T GetInterface<T>(this GameObject gameObject) where T : class
		{
			return gameObject.GetInterfaces<T>().Single();
		}

		public static IEnumerable<T> GetInterfaces<T>(this GameObject gameObject) where T : class
		{
			if (!typeof(T).IsInterface)
			{
				Debug.LogError(string.Format("[{0}] is not an interface.", typeof(T)));
				return Enumerable.Empty<T>();
			}

			return gameObject.GetComponents<Component>().OfType<T>();
		}

		public static string GetFullName(this GameObject gameObject)
		{
			return GetFullName(gameObject.transform);
		}

		public static string GetFullName(this Transform transform)
		{
			var sb = new StringBuilder ();
			while (transform != null)
			{
				sb.Append("/").Append(transform.name);
				transform = transform.parent;
			}
			return sb.ToString();
		}

		public static string GetClonelessName(this GameObject gameObject)
		{
			const string CLONE_MONIKER = "(Clone)";
			return gameObject.name.Replace(CLONE_MONIKER, string.Empty);
		}

		public static GameObject SetLayerWithChildren(this GameObject gameObject, int changeToLayer)
		{
			Debug.Log(gameObject.name + " [" + gameObject.layer + "] SetLayerRecursively " + changeToLayer, gameObject);

			gameObject.layer = changeToLayer;
			foreach (Transform child in gameObject.transform)
				child.gameObject.layer = changeToLayer;
			return gameObject;
		}

		public static GameObject SetLayerRecursively(this GameObject gameObject, int changeToLayer)
		{
			Debug.Log(gameObject.name + " [" + gameObject.layer + "] SetLayerRecursively " + changeToLayer, gameObject);

			var transforms = gameObject.GetComponentsInChildren<Transform>();
			foreach (var t in transforms)
				t.gameObject.layer = changeToLayer;
			return gameObject;
		}
	}
}
