using UnityEngine;
using System.Collections;

public static class HashtableExtensions
{
	public static T GetValue<T>(this Hashtable hash, string key)
	{
		return hash.ContainsKey(key) ? (T)hash [key] : default(T);
	}

	public static bool TryGetValue<T>(this Hashtable hash, string key, out T value)
	{
		value = GetValue<T>(hash, key);
		return hash.ContainsKey(key);
	}

	public static Hashtable MergeAdd(this Hashtable hash, string key, ArrayList ar)
	{
		Debug.Log(string.Format("{0} MergeAdd {1} {2}", hash, key, ar));

		if (!hash.ContainsKey(key))
		{
			//if the key isn't present, this is easy.
			Debug.Log("key " + key + " not present; adding.");
			hash.Add(key, ar);
			return hash;
		}

		//so we do have this key, and we need to merge into the values
		var tmp = hash.GetValue<ArrayList>(key);
		if (tmp == null)
		{
			Debug.LogError("value not an arraylist");
			return hash;
		}

		//remove the old entry
		hash.Remove(key);

		//add any new values to the arraylist
		foreach (var item in ar)
		{
			if (!tmp.Contains(item))
			{
				tmp.Add(item);
			}
		}

		//add the entry back
		hash.Add(key, tmp);
		return hash;
	}
}
