//from http://orbcreation.com/orbcreation/page.orb?1099
using UnityEngine;
using System.Text;

public static class RectTransformExtensions
{
	public static void SetDefaultScale(this RectTransform rectTransform)
	{
		rectTransform.localScale = Vector3.one;
	}
	
	public static void SetPivotAndAnchors(this RectTransform rectTransform, Vector2 aVec)
	{
		rectTransform.pivot = aVec;
		rectTransform.anchorMin = aVec;
		rectTransform.anchorMax = aVec;
	}
	
	public static Vector2 GetSize(this RectTransform rectTransform)
	{
		return rectTransform.rect.size;
	}
	
	public static float GetWidth(this RectTransform rectTransform)
	{
		return rectTransform.rect.width;
	}
	
	public static float GetHeight(this RectTransform rectTransform)
	{
		return rectTransform.rect.height;
	}
	
	public static void SetPositionOfPivot(this RectTransform rectTransform, Vector2 localPosition)
	{
		rectTransform.localPosition = new Vector3(localPosition.x, localPosition.y, rectTransform.localPosition.z);
	}
	
	public static void SetLeftBottomPosition(this RectTransform rectTransform, Vector2 localPosition)
	{
		rectTransform.localPosition = new Vector3(localPosition.x + (rectTransform.pivot.x * rectTransform.rect.width), localPosition.y + (rectTransform.pivot.y * rectTransform.rect.height), rectTransform.localPosition.z);
	}
	
	public static void SetLeftTopPosition(this RectTransform rectTransform, Vector2 localPosition)
	{
		rectTransform.localPosition = new Vector3(localPosition.x + (rectTransform.pivot.x * rectTransform.rect.width),
		                                          localPosition.y - ((1f - rectTransform.pivot.y) * rectTransform.rect.height),
		                                          rectTransform.localPosition.z);
	}
	
	public static void SetRightBottomPosition(this RectTransform rectTransform, Vector2 localPosition)
	{
		rectTransform.localPosition = new Vector3(localPosition.x - ((1f - rectTransform.pivot.x) * rectTransform.rect.width),
		                                          localPosition.y + (rectTransform.pivot.y * rectTransform.rect.height),
		                                          rectTransform.localPosition.z);
	}
	
	public static void SetRightTopPosition(this RectTransform rectTransform, Vector2 localPosition)
	{
		rectTransform.localPosition = new Vector3(localPosition.x - ((1f - rectTransform.pivot.x) * rectTransform.rect.width),
		                                          localPosition.y - ((1f - rectTransform.pivot.y) * rectTransform.rect.height),
		                                          rectTransform.localPosition.z);
	}
	
	public static void SetSize(this RectTransform rectTransform, Vector2 size)
	{
		Vector2 oldSize = rectTransform.rect.size;
		Vector2 deltaSize = size - oldSize;
		rectTransform.offsetMin = rectTransform.offsetMin - new Vector2(deltaSize.x * rectTransform.pivot.x,
		                                                                deltaSize.y * rectTransform.pivot.y);
		rectTransform.offsetMax = rectTransform.offsetMax + new Vector2(deltaSize.x * (1f - rectTransform.pivot.x),
		                                                                deltaSize.y * (1f - rectTransform.pivot.y));
	}
	
	public static void SetWidth(this RectTransform rectTransform, float width)
	{
		SetSize(rectTransform, new Vector2(width, rectTransform.rect.size.y));
	}
	
	public static void SetHeight(this RectTransform rectTransform, float height)
	{
		SetSize(rectTransform, new Vector2(rectTransform.rect.size.x, height));
	}
	
	public static string Dump(this RectTransform rectTransform, string prefix = null)
	{
		if (rectTransform == null)
		{
			Debug.LogError("No RectTransform component passed in but it's required for logging.");
		}
		else
		{
			var sb = new StringBuilder();
			if (!string.IsNullOrEmpty(prefix))
			{
				sb.AppendFormat("[{0}] ", prefix);
			}
			sb.AppendFormat("{0}+RectTransform: ", rectTransform.name);
			sb.AppendFormat(" {0}: {1}", "anchoredPosition", rectTransform.anchoredPosition).AppendLine();
			sb.AppendFormat(" {0}: {1}", "anchorMax", rectTransform.anchorMax).AppendLine();
			sb.AppendFormat(" {0}: {1}", "anchorMin", rectTransform.anchorMin).AppendLine();
			sb.AppendFormat(" {0}: {1}", "hasChanged", (rectTransform.hasChanged ? "YES" : "NO")).AppendLine();
			sb.AppendFormat(" {0}: {1}", "offsetMax", rectTransform.offsetMax).AppendLine();
			sb.AppendFormat(" {0}: {1}", "offsetMin", rectTransform.offsetMin).AppendLine();
			sb.AppendFormat(" {0}: {1}", "pivot", rectTransform.pivot).AppendLine();
			sb.AppendFormat(" {0}: {1}", "rect", rectTransform.rect).AppendLine();
			sb.AppendFormat(" {0}: {1}", "sizeDelta", rectTransform.sizeDelta).AppendLine();
			sb.AppendFormat(" {0}: {1}", "GetSiblingIndex()", rectTransform.GetSiblingIndex()).AppendLine();
			
			var localCorners = new Vector3[4];
			rectTransform.GetLocalCorners(localCorners);
			sb.AppendFormat("{0} = {1}, {2}, {3}, {4}", "GetLocalCorners()", localCorners[0], localCorners[1], localCorners[2], localCorners[3]).AppendLine();
			
			var worldCorners = new Vector3[4];
			rectTransform.GetWorldCorners(worldCorners);
			sb.AppendFormat("{0} = {1}, {2}, {3}, {4}", "GetWorldCorners()", worldCorners[0], worldCorners[1], worldCorners[2], worldCorners[3]).AppendLine();
			
			return sb.ToString();
		}
		
		return null;
	}
}
