using UnityEngine;
using System.Globalization;
using System.Text.RegularExpressions;

public static class StringExtensions
{
	const string FOWARD = @"/";
	const string BACK = @"\";

	public static bool IsNullOrEmpty(this string input)
	{
		return string.IsNullOrEmpty(input);
	}

	// TODO this should not be an extension method for the string class...
	// reference: https://tools.ietf.org/html/draft-hammer-http-token-auth-01
	public static string FormatToken(this string token)
	{
		return string.Format("Token token=\"{0}\"", token);
	}

	public static string ReplaceBackWithForwardSlashes(this string value)
	{
		return value.Replace(BACK, FOWARD);
	}

	public static string ReplaceFowardWithBackSlashes(this string value)
	{
		return value.Replace(FOWARD, BACK);
	}

	/// <summary>
	/// Attemps to convert a hexadecimal string to a Color. Supports (AA)RRGGBB formats.
	/// </summary>
	/// <returns>The color.</returns>
	/// <param name="value">Value.</param>
	public static Color ToColor(this string value)
	{
		var match = Regex.Match(value, RegexPatterns.HEX_COLOR);
		if (match.Success)
		{
			const NumberStyles NUMBER_STYLE = NumberStyles.HexNumber;
			value = match.Value;

			byte a, r, g, b;

			switch (value.Length)
			{
				case 8:
					a = byte.Parse(value.Substring(0, 2), NUMBER_STYLE);
					r = byte.Parse(value.Substring(2, 2), NUMBER_STYLE);
					g = byte.Parse(value.Substring(4, 2), NUMBER_STYLE);
					b = byte.Parse(value.Substring(6, 2), NUMBER_STYLE);
					return new Color32(r, g, b, a);

				case 6:
					a = 255;
					r = byte.Parse(value.Substring(0, 2), NUMBER_STYLE);
					g = byte.Parse(value.Substring(2, 2), NUMBER_STYLE);
					b = byte.Parse(value.Substring(4, 2), NUMBER_STYLE);
					return new Color32(r, g, b, a);
			}
		}
		return GetColorFromLiteral(value);
	}

	static Color GetColorFromLiteral(string value)
	{
		// support some basic colors without convering from hex
		switch (value.ToLower())
		{
			case "black":
				return Color.black;
			case "blue":
				return Color.blue;
			case "clear":
			case "transparent":
				return Color.clear;
			case "cyan":
				return Color.cyan;
			case "gray":
				return Color.gray;
			case "green":
				return Color.green;
			case "grey":
				return Color.grey;
			case "magenta":
				return Color.magenta;
			case "red":
				return Color.red;
			case "white":
				return Color.white;
			case "yellow":
				return Color.yellow;

		// css colors (there's a lot, this is an example; the rest can be found here: http://www.w3schools.com/colors/colors_names.asp)
			case "aliceblue":
				return "#F0F8FF".ToColor();
			case "antiquewhite":
				return "#FAEBD7".ToColor();
			case "aqua":
				return "#00FFFF".ToColor();
			case "aquamarine":
				return "#7FFFD4".ToColor();
			case "azure":
				return "#F0FFFF".ToColor();
			case "beige":
				return "#F5F5DC".ToColor();
			case "bisque":
				return "#FFE4C4".ToColor();
			case "blanchedalmond":
				return "#FFEBCD".ToColor();
			case "blueviolet":
				return "#8A2BE2".ToColor();
			case "brown":
				return "#A52A2A".ToColor();
			case "burlywood":
				return "#DEB887".ToColor();
			case "whitesmoke":
				return "#F5F5F5".ToColor();
			case "yellowgreen":
				return "#9ACD32".ToColor();
			case "wheat":
				return "#F5DEB3".ToColor();
			case "violet":
				return "#EE82EE".ToColor();
			default:
				if (Debug.isDebugBuild)
					Debug.LogWarning(string.Format("Value [{0}] could not be determined. Returning Color.clear", value));
				return Color.clear;
		}
	}

	// we only support converting 6 and 8 digit values anyway,
	// so limit to one or the other, while including the hash symbol.
	public static bool CanConvertToHex(this string value)
	{
		var length = value.Length;
		return value.Contains("#") && (length == 7 || length == 9);
	}
}

namespace UnityEditor
{
	public static class UnityEditorStringExtensions
	{
		public static string GetRelativeAssetPath(this string path)
		{
			return path.ReplaceBackWithForwardSlashes()
				.Replace(Application.dataPath, "Assets");
		}
	}
}
