﻿public class Observable<T>
{
	public delegate void ValueChanged(T newValue);
	
	#region Events
	public event ValueChanged OnValueChanged;
	#endregion
	
	#region Constructors
	public Observable()
	{
		backingValue = default(T);
	}

	public Observable(T initialValue)
	{
		backingValue = initialValue;
	}
	#endregion
	
	#region Properties
	public T Value
	{
		get { return backingValue; }
		set
		{
			var raiseValueChangedEvent = OnValueChanged != null && (backingValue == null
			                                                        ? value != null
			                                                        : !backingValue.Equals(value));
			backingValue = value;

			if (raiseValueChangedEvent)
			{
				OnValueChanged(value);
			}
		}
	}
	#endregion

	#region Operators
	public static implicit operator T(Observable<T> observable)
	{
		return observable.Value;
	}
	#endregion

	#region Actions
	public override bool Equals(object obj)
	{
		return Value.Equals(obj);
	}

	public override int GetHashCode()
	{
		return Value.GetHashCode();
	}

	public override string ToString()
	{
		return Value.ToString();
	}
	#endregion

	#region Private
	T backingValue;
	#endregion
}
