﻿using UnityEngine;

/// <summary>
/// Calls `setSoftInputMode` in the UI thread to update the adjustment settings to *not* resize or shift the game window.
/// </summary>
public class AndroidKeyboardConfig : MonoBehaviour
{
	#if UNITY_ANDROID
	void Start()
	{
		if (Application.platform != RuntimePlatform.Android)
		{
			return;
		}

		AndroidJNI.AttachCurrentThread();
		AndroidJNI.PushLocalFrame(0);
		try
		{
			using (AndroidJavaClass jcUnityPlayer = new AndroidJavaClass ("com.unity3d.player.UnityPlayer"))
			using (AndroidJavaObject joActivity = jcUnityPlayer.GetStatic<AndroidJavaObject>("currentActivity"))
			{
				joActivity.Call("runOnUiThread", new AndroidJavaRunnable (RunOnUiThread));
			}
		}
		catch (System.Exception ex)
		{
			Debug.LogError(ex.Message);
		}
		finally
		{
			AndroidJNI.PopLocalFrame(System.IntPtr.Zero);
		}
	}

	void RunOnUiThread()
	{
		using (AndroidJavaClass jcUnityPlayer = new AndroidJavaClass ("com.unity3d.player.UnityPlayer"))
		using (AndroidJavaObject joActivity = jcUnityPlayer.GetStatic<AndroidJavaObject> ("currentActivity"))
		using (AndroidJavaObject joWindow = joActivity.Call<AndroidJavaObject> ("getWindow")) 
		{
			// http://developer.android.com/reference/android/view/WindowManager.LayoutParams.html#SOFT_INPUT_ADJUST_NOTHING
			joWindow.Call ("setSoftInputMode", 48);
		}
	}
	#endif
}
